/*============================================================================
| 2008 - David Duvivier
|
| Operating System Basic Definitions and Tools
#############################################################################
#This program is free software; you can redistribute it and/or modify       #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#############################################################################
============================================================================*/

/*----------------------------------------------------------------------------
|			D_E_F_I_N_E_S  (debug mode, check mode, ...)
----------------------------------------------------------------------------*/

/* Mode dbuggage (affiche des messages d'tats des process) : */
/* Remplacez "undef" par "define" en ci-dessous pour passer en */
/* mode dbuggage (affiche "plein de messages") et inversement */
#undef DD_DEBUG			/* Debug mode ? */

#define DD_NOCHECK

/*----------------------------------------------------------------------------
| Header files
----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <syslog.h>
#include <grp.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <sys/resource.h>
#include <sys/queue.h>

#if !defined(__STDC__)
#include <malloc.h>
#endif

/*----------------------------------------------------------------------------
| Constants
----------------------------------------------------------------------------*/
#define DD_VERSION   "0.1"                /* Version 0.1 */
#define DD_COPYRIGHT "%% 2008 - David Duvivier.\n\
%% Operating System Basic Definitions and Tools."

#ifndef TRUE
#define FALSE         0
#define TRUE          !(FALSE)
#endif
#define DD_OK            0
#define DD_ERROR         !(OK)
#define DD_UNSPECIFIED  -1

#define semaphore0	0
#define semaphore1	1
#define semaphore2	2
#define semaphore3	3
#define semaphore4	4
#define semaphore5	5
#define semaphore6	6
#define semaphore7	7
#define semaphore8	8
#define semaphore9	9

/*----------------------------------------------------------------------------
| Pseudo-functions
----------------------------------------------------------------------------*/

#ifdef DD_DEBUG
#define DD_TRACE(A)             puts(A); fflush(stdout)
#define DD_TRACE2(A,B)          printf(A,B); puts(""); fflush(stdout)
#define DD_TRACE3(A,B,C)        printf(A,B,C); puts(""); fflush(stdout)
#define DD_TRACE4(A,B,C,D)      printf(A,B,C,D); puts(""); fflush(stdout)
#else
#define DD_TRACE(A)
#define DD_TRACE2(A,B)
#define DD_TRACE3(A,B,C)
#define DD_TRACE4(A,B,C,D)
#endif

/* Affiche un message d'information sur stderr */
#define DD_pidmsg2(a, b)	\
	fprintf(stderr, "%-12s:pid=%d:%s\n", a, getpid(), b);\
	fflush(stderr);

#define DD_pidmsg3(a, b, c)	\
	fprintf(stderr, "%-12s:pid=%d:%s%d)\n", a, getpid(), b, c);\
	fflush(stderr);

/* Affiche un message d'erreur sur stderr */
#define DD_piderr2(a, b)	\
	fprintf(stderr, "%-12s:pid=%d:%s:%s\n", a, getpid(), strerror(errno), b);\
	fflush(stderr);

/* Affiche un message d'information si DD_DEBUG est dfini */
/* sinon cette macro ne fait rien (!) */
#ifdef DD_DEBUG
#define DD_pidmsgdebug2(a, b)	\
	fprintf(stderr, "%-12s:pid=%d:%s\n", a, getpid(), b);\
	fflush(stderr);
#define DD_pidmsgdebug3(a, b, c)	\
	fprintf(stderr, "%-12s:pid=%d:%s%d)\n", a, getpid(), b, c);\
	fflush(stderr);
#else
#define DD_pidmsgdebug2(a, b)
#define DD_pidmsgdebug3(a, b, c)
#endif

/*------------------------ random number in [0..1] -------------------------*/
#define DD_RAND_FRAC() ((double)rand()/(double)(RAND_MAX))

/*-------------------- random number in domain inf..sup --------------------*/
/* Gnre un nombre pseudo-alatoire (entier) entre inf et sup  (entiers) */
/* (cf man de rand() et srand()) inf et sup sont tels que inf < sup */
#define DD_RAND_DOM(inf, sup) \
	((inf) + (int) ((double)(sup) * (rand() / (RAND_MAX + (double)(inf)))))

/*------------------------------- random duration --------------------------*/
/* Gnre un nombre alatoire (entier) entre 1 et 200000 + pid */
/* (soit un "bruit" de 2/10 sec environ) */
/* Le pid est utilis pour ajouter un bruit supplmentaire (3/100 sec environ) */
#define DD_RAND_PID \
	DD_RAND_DOM(1,200000+getpid())
/* Gnre un dlai entre inf et inf + 200000 + pid */
#define DD_RAND_USLEEP(inf) \
	usleep(inf + DD_RAND_PID)

/*------------------------------- random bit -------------------------------*/
#define DD_RAND_BIT()  ((DD_RAND_FRAC()>=.5)? 1 : 0 )

/*------------------------------- min and max ------------------------------*/
#define DD_MIN(a,b) (((a) < (b)) ? (a) : (b))
#define DD_MAX(a,b) (((a) > (b)) ? (a) : (b))

/*-------------------------------- Swapping --------------------------------*/
#define DD_iswap(a, b) {int tmp; tmp = *(a); *(a) = *(b); *(b) = tmp;}

/*----------------------------------------------------------------------------
| Function prototypes
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
| Initialisation des structures de donnes de DDTools
| ATTENTION doit tre effectue une seule fois par le processus pre
|           AVANT DE CREER LES PROCESSUS FILS via fork()
| Les paramtres sont les suivants :
| - argc, argv        : argc et argv de la procdure principale main()
| - Nombre_Semaphores : nombre de smaphores dans le tableau de smaphores
----------------------------------------------------------------------------*/
extern void DD_init(int argc, char *argv[], int Nombre_Semaphores);

/*----------------------------------------------------------------------------
| Libration des ressources systme utilises par DDTools
| ATTENTION doit tre effectue une seule fois par le dernier processus actif
|	    (i.e. classiquement le processus pre)
----------------------------------------------------------------------------*/
extern void DD_close(void);

/*----------------------------------------------------------------------------
| Bloque le processus pre tant qu'au moins un fils est actif.
----------------------------------------------------------------------------*/
extern void DD_waitallchilds(void);

/*----------------------------------------------------------------------------
| Initialisation du gnrateur alatoire 
| ATTENTION doit tre effectu par chaque processus (pre, fils...)
----------------------------------------------------------------------------*/
extern void DD_initrandom(void);

/*----------------------------------------------------------------------------
| Fixe la valeur initiale du smaphore N"sem"  la valeur "val"
----------------------------------------------------------------------------*/
extern void DD_sem_setvalue(int sem, int val);

/*----------------------------------------------------------------------------
| Lit "la valeur" du smaphore numro "sem" du au tableau de smaphores
| ATTENTION : SEULES LES VALEURS >= 0 SONT RENVOYEES (i.e. s'il y a des
| processus bloqus par le smaphores, la valeur retroune sera "0" et non
| une valeur ngative relative au nombre de processus bloqus
----------------------------------------------------------------------------*/
extern int DD_sem_getvalue(int sem);

/*----------------------------------------------------------------------------
| Demande l'accs  la ressource protge par le smaphore N"sem" :
| quivaut  P(sem)
| quivaut  Down(sem)
----------------------------------------------------------------------------*/
#define P(sem)		DD_sem_askresource(sem)
#define Down(sem)	DD_sem_askresource(sem)
extern int DD_sem_askresource(int sem);

/*----------------------------------------------------------------------------
| Libre l'accs  la ressource protge par le smaphore N"sem" :
| quivaut  V(sem)
| quivaut  Up(sem)
----------------------------------------------------------------------------*/
#define V(sem)		DD_sem_releaseresource(sem)
#define Up(sem)		DD_sem_releaseresource(sem)
extern int DD_sem_releaseresource(int sem);

/*----------------------------------------------------------------------------
| Incrmente une variable partage "simule"  l'aide du smaphore N"sem"
| ATTENTION ceci est fortement dconseill et juste destin  simplifier le
| code pour les L2 informatique
----------------------------------------------------------------------------*/
#define Incremente(sem)	DD_sem_releaseresource(sem)

/*----------------------------------------------------------------------------
| Dcrmente une variable partage "simule"  l'aide du smaphore N"sem"
| ATTENTION ceci est fortement dconseill et juste destin  simplifier le
| code pour les L2 informatique
| ATTENTION si la valeur devient ngative le processus courant est bloqu !!!
----------------------------------------------------------------------------*/
#define Decremente(sem)	DD_sem_askresource(sem)
