#!/usr/bin/python

import pygtk
pygtk.require('2.0')
import gtk

class area(gtk.DrawingArea):
    def __init__(self, size, grid, gui):
        gtk.DrawingArea.__init__(self)
        self.set_events(gtk.gdk.BUTTON_PRESS_MASK|gtk.gdk.BUTTON_RELEASE_MASK|gtk.gdk.POINTER_MOTION_MASK)

        self.margin = 5

        self.size = size
        self.grid = grid
        self.set_size_request(180,180)
        self.gui = gui
        self.gui.window.connect("configure_event" , self.resize_event)
        self.connect("expose-event", self.area_expose_cb)
        self.connect("button-release-event", self.button_event, self.gui)
        self.connect("motion-notify-event", self.mouse_move)

        self.last = None
        self.over = None

    def mouse_move(self, widget, event):
        x, y = event.get_coords()
        i = int(x / self.t_x)
        j = int(y / self.t_y)

        if i >= 0 and i <= self.size + 1 and j >= 0 and j <= self.size + 1 and i != j \
            and i + j != self.size + 1 \
            and (i == 0 or i == self.size + 1 or j == 0 or j == self.size + 1):
            self.over = (i, j)
            self.drawRing()

    def posToMove(self, pos):
        x, y = pos

        number = 0
        position = ""

        if x == 0 or x == self.size + 1:
            number = y - 1
        else:
            number = x - 1

        if x == 0:
            position = "left"
        elif x == self.size + 1:
            position = "right"
        elif y == 0:
            position = "top"
        elif y == self.size + 1:
            position = "bottom"

        return {"number": number, "position": position}

    def moveToPos(self, move):
        p = None
        if move["position"] == "left":
            p = (0, int(move["number"]) + 1)
        elif move["position"] == "right":
            p = (self.grid.size + 1, int(move["number"]) + 1)
        elif move["position"] == "top":
            p = (int(move["number"]) + 1, 0)
        elif move["position"] == "bottom":
            p = (int(move["number"]) + 1, self.grid.size + 1)
        return p

    def draw(self, move = None):
        if move != None:
            self.last = self.moveToPos(move)

        m = self.margin

        fen_x,fen_y = self.window.get_size()

        #calcul la taille des rectangles+1
        t_x = (fen_x - 2 * m) / (self.size + 2)
        t_y = (fen_y - 2 * m) / (self.size + 2)
        self.t_x = t_x
        self.t_y = t_y

        #Et on les affiche !
        for y in range(self.size):
            for x in range(self.size):
                if self.grid[x+y*self.size] == -1 or self.grid[x+y*self.size] == -2:
                    self.gc.foreground = self.get_colormap().alloc_color(0, 65535, 0)
                if self.grid[x+y*self.size] == 1 or  self.grid[x+y*self.size] == 2:
                    self.gc.foreground = self.get_colormap().alloc_color(65535, 0, 0)

                if self.grid[x+y*self.size] == -1 or self.grid[x+y*self.size] == 1:
                    self.window.draw_rectangle(self.gc, True, m+(x+1)*t_x, m+(y+1)*t_y, t_x-1, t_y-1)
                else:
                    self.window.draw_rectangle(self.gc, True, m+(x+1)*t_x, m+(y+1)*t_y, t_x-1, t_y-1)
                    self.gc.foreground = self.get_colormap().alloc_color(0, 0, 0)
                    self.window.draw_rectangle(self.gc, True, m+(x+1)*t_x+t_x/4, m+(y+1)*t_y+t_y/4, t_x/2, t_y/2)
        self.drawRing()


    def drawRing(self):
        m = self.margin
        t_x = self.t_x
        t_y = self.t_y
        for y in range(self.size + 2):
            for x in range(self.size + 2):
                if x != y and x + y != self.size + 1 and (x == 0 or x == self.size + 1 or y == 0 or y == self.size + 1):
                    self.gc.foreground = self.get_colormap().alloc_color(65535, 65535, 65535)
                    self.window.draw_rectangle(self.gc, True, m+(x)*t_x, m+(y)*t_y, t_x-1, t_y-1)

                self.gc.foreground = self.get_colormap().alloc_color(0, 0, 0)
                self._drawTriangle(x, y)

        # dessin du triangle sous le pointeur
        if self.over != None:
            self.gc.foreground = self.get_colormap().alloc_color(40000, 40000, 40000)
            self._drawTriangle(self.over[0], self.over[1])

        # dessin du triangle du dernier mouvement
        if self.last != None:
            self.gc.foreground = self.get_colormap().alloc_color(0, 0, 65535)
            self._drawTriangle(self.last[0], self.last[1])

    def _drawTriangle(self, x, y):
        m = self.margin
        t_x = self.t_x
        t_y = self.t_y
        if x == 0 and y > 0  and y < self.size + 1:
            self.window.draw_polygon(self.gc, True, ((m+(x)*t_x, m+(y)*t_y), (m+(x)*t_x, m+(y)*t_y+t_y), (m+(x)*t_x+t_x/2, m+(y)*t_y+t_y/2)))

        if x == self.size + 1 and y > 0  and y < self.size + 1:
            self.window.draw_polygon(self.gc, True, ((m+(x)*t_x+t_x, m+(y)*t_y), (m+(x)*t_x+t_x, m+(y)*t_y+t_y), (m+(x)*t_x+t_x/2, m+(y)*t_y+t_y/2)))

        if y == 0 and x > 0  and x < self.size + 1:
            self.window.draw_polygon(self.gc, True, ((m+(x)*t_x, m+(y)*t_y), (m+(x)*t_x+t_x, m+(y)*t_y), (m+(x)*t_x+t_x/2, m+(y)*t_y+t_y/2)))

        if y == self.size + 1 and x > 0  and x < self.size + 1:
            self.window.draw_polygon(self.gc, True, ((m+(x)*t_x, m+(y)*t_y+t_y), (m+(x)*t_x+t_x, m+(y)*t_y+t_y), (m+(x)*t_x+t_x/2, m+(y)*t_y+t_y/2)))

    def area_expose_cb(self, area, event):
        self.draw()
        return True

    def setGC(self):
        self.gc = gtk.gdk.GC(self.window)

    def resize_event(self, widget, event):
        self.draw()


    def button_event(self, widget, event, gui):
        x, y = event.get_coords()
        i = int(x / self.t_x)
        j = int(y / self.t_y)

        if i >= 0 and i <= self.size + 1 and j >= 0 and j <= self.size + 1 and i != j \
            and i + j != self.size + 1 \
            and (i == 0 or i == self.size + 1 or j == 0 or j == self.size + 1):
            self.last = (i, j)
            self.drawRing()
            try:
                gui.setMove(self.posToMove(self.last))
            except:
                print "No handler to set move!"
