/*----------------------------------------------------------------------*/
/* Fichier : consommateur2tn.c                                          */
/* Auteur  : D. Duvivier - LIL/ULCO                                     */
/* Date    : 14/10/08                                                   */
/* Objectif: TP N2 (correction) M2/I2L Systme Rseaux Scurit        */
/*           Test de deux tubes nomms entre deux processus "lourds"    */
/*           Ce programme est le consommateur de donnes.               */
/*           ===========================================                */
/*           Il ne doit pas tre lanc directement depuis le shell      */
/*           ====================================================       */
/*           (le producteur s'en charge ds qu'il est initialis/pret). */
/*----------------------------------------------------------------------*/
/* ATTENTION: CETTE VERSION DU CONSOMMATEUR NE FONCTIONNE PAS !         */
/* Je suis d'une telle maladresse que j'ai commis "une impardonnable    */
/* erreur par inadvertance" ;-) !                                       */
/* A vous de la localiser et de la corriger ... facile !                */
/*----------------------------------------------------------------------*/
/* Remarque : les appels "fflush(stdout)" ne servent  rien si ce n'est */
/* pour s'assurer que les messages soient bien affichs immdiatement   */
/* en vitant les dlais ds  la buffrisation des e/s.                */
/* Remarque2 : les appels "fflush(fichier1)" et "fflush(fichier2)" sont */
/* ncessaires pour utiliser les tubes comme moyen de synchronisation   */
/* sinon le systme de bufferisation retarde les critures effectives   */
/* dans les tubes et on a un deadlock (les 2 process sont en attente    */
/* blocante sur "fgets"), ceci n'est pas conseill en gnral car les   */
/* E/S sont moins performantes sans le systme de bufferisation mais il */
/* est ncessaire de le courcircuiter (soit en utilisant des E/S non    */
/* buffrises (i.e. de bas niveau), soit en vidant les buffers aprs   */
/* chaque criture (via fflush)). Ici on s'en sort avec fflush, mais    */
/* mais lorqu'il s'agit d'crire des drivers ou du code "sensible"      */
/* il est ncessaire de programmer a beaucoup plus "proprement".       */
/* L'objectif ici est de vous montrer les problmes lis  la           */
/* bufferisation des E/S (improprement utilises pour faire de la       */
/* synchronisation).                                                    */
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>

/* Variables globales -> a viter/limiter normalement (!) */
char *nom_tube1="prodcons2tube"; /* tube producteur -> consommateur */
char *nom_tube2="consprod2tube"; /* tube consommateur -> producteur */
FILE *fichier1;
int fd1;
FILE *fichier2;
int fd2;
char buffer[128];

int main(void) {
  int i = 0; /* Indice de boucle */

  printf("Consommateur (pid=%d) : Lancement du processus consommateur...\n", getpid());

  printf("Consommateur (pid=%d) : Attente de lecture sur le tube 1...\n", getpid());

  /* Ouverture en lecture du tube 1, attention l'ouverture est blocante */
  /* si pas d'metteur  l'autre bout du tube ! */
  fd1 = open(nom_tube1, O_RDONLY);
  fichier1 = fdopen(fd1, "r");
  printf("Consommateur (pid=%d) : Tube 1 ouvert...\n", getpid());

  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
  /* En ce point, une synchronisation entre le producteur et le         */
  /* consommateur a eu lieu : ds que le tube (vide pour l'instant)     */
  /* comporte (au moins) un metteur et (au moins) un rcepteur, ces    */
  /* processus sont dbloqus (ici seulement deux processus prvus).    */
  /* s'il y a dj des choses dans le tube, le rcepteur est            */
  /* immdiatement dbloqu.                                            */
  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

  printf("Consommateur (pid=%d) : Ouverture en criture sur le tube 2...\n", getpid());

  /* Ouverture en criture du tube 2, attention l'ouverture est blocante */
  /* si pas de lecteur  l'autre bout du tube ! */
  fd2 = open(nom_tube2, O_WRONLY);
  fichier2 = fdopen(fd2, "w");
  printf("Consommateur (pid=%d) : Tube 2 ouvert, par  lancer...\n", getpid());
  fflush(stdout);

  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
  /* En ce point, une seconde synchronisation entre le producteur et le */
  /* consommateur a eu lieu : ds que le tube (vide pour l'instant)     */
  /* comporte (au moins) un metteur et (au moins) un rcepteur, ces    */
  /* processus sont dbloqus (ici seulement deux processus prvus).    */
  /* s'il y a dj des choses dans le tube, le rcepteur est            */
  /* immdiatement dbloqu.                                            */
  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

  /* Seconde partie du "ping pong" : le consommateur rceptionne le */
  /* ping et renvoie un pong en retour ! */

  sleep(3);

  printf("Consommateur (pid=%d) : Attente de la premire rception sur tube 1 ...\n", getpid());
  fflush(stdout);

  /* ATTENTION : ci-dessous le fgets est "blocant" sur un tube ! */
  /* Il reste bloqu tant qu'il existe une possibilit de lire qqchose */
  /* c'est--dire qu'un second processus  un accs en criture */
  /* ds que ce processus ferme son descripteur fgets se dbloque et */
  /* NULL est retourn ; le second cas est le cas o il y a des caractres */
  /* qui arrivents dans le tube, auquel cas le fgets se dbloque et */
  /* retourne le nombre de caractres lus */
  /* -> ATTENTION aux DEADLOCKS avec fgets ! */
  while (fgets(buffer, 128, fichier1) != NULL) {
    printf("Consommateur (pid=%d) : Reoit %s", getpid(), buffer);
    fflush(stdout);
    i++;
    printf("Consommateur (pid=%d) : Emet PONG %d sur tube 2 destin au producteur\n", getpid(), i);
    fflush(stdout);
    fprintf(fichier2, "Consommateur (pid=%d) : PONG %d sur tube 2 vers producteur\n", getpid(), i);
    /* Vous pouvez tester l'effet du fflush en commentant la ligne suivante */
    /* --> sans flush, pas d'criture effective dans le tube -> blocage */
    fflush(fichier2);
    sleep(1); /* Petit dlai pour ralentir le flux */
    printf("Consommateur (pid=%d) : Attente de rception sur tube 1 ...\n", getpid());
    fflush(stdout);
  }

  printf("Consommateur (pid=%d) : Fin du ping pong !\n", getpid());
  fflush(stdout);
  fclose(fichier2);
  close(fd2);
  fclose(fichier1);
  close(fd1);
  printf("Consommateur (pid=%d) : Fin du processus consommateur...\n", getpid());
  fflush(stdout);
  return(EXIT_SUCCESS);
}
