/*----------------------------------------------------------------------*/
/* Fichier : producteur2tn.c                                            */
/* Auteur  : D. Duvivier - LIL/ULCO                                     */
/* Date    : 14/10/08                                                   */
/* Objectif: TP N2 (correction) M2/I2L Systme Rseaux Scurit        */
/*           Test de deux tubes nomms entre deux processus "lourds"    */
/*           Ce programme est le producteur de donnes.                 */
/*           Il se charge de lancer le consommateur de donnes          */
/*           (qui doit avoir ete compile prealablement !)               */
/*----------------------------------------------------------------------*/
/* ATTENTION: CETTE VERSION DU PRODUCTEUR NE FONCTIONNE PAS !           */
/* Je suis d'une telle maladresse que j'ai commis "une impardonnable    */
/* erreur par inadvertance" ;-) !                                       */
/* A vous de la localiser et de la corriger ... facile !                */
/*----------------------------------------------------------------------*/
/* Remarque : les appels "fflush(stdout)" ne servent  rien si ce n'est */
/* pour s'assurer que les messages soient bien affichs immdiatement   */
/* en vitant les dlais ds  la buffrisation des e/s.                */
/* Remarque2 : les appels "fflush(fichier1)" et "fflush(fichier2)" sont */
/* ncessaires pour utiliser les tubes comme moyen de synchronisation   */
/* sinon le systme de bufferisation retarde les critures effectives   */
/* dans les tubes et on a un deadlock (les 2 process sont en attente    */
/* blocante sur "fgets"), ceci n'est pas conseill en gnral car les   */
/* E/S sont moins performantes sans le systme de bufferisation mais il */
/* est ncessaire de le courcircuiter (soit en utilisant des E/S non    */
/* buffrises (i.e. de bas niveau), soit en vidant les buffers aprs   */
/* chaque criture (via fflush)). Ici on s'en sort avec fflush, mais    */
/* mais lorqu'il s'agit d'crire des drivers ou du code "sensible"      */
/* il est ncessaire de programmer a beaucoup plus "proprement".       */
/* L'objectif ici est de vous montrer les problmes lis  la           */
/* bufferisation des E/S (improprement utilises pour faire de la       */
/* synchronisation).                                                    */
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>

/* Variables globales -> a viter/limiter normalement (!) */
char *nom_tube1="prodcons2tube"; /* tube producteur -> consommateur */
char *nom_tube2="consprod2tube"; /* tube consommateur -> producteur */
FILE *fichier1;
int fd1;
FILE *fichier2;
int fd2;
char buffer[128];
int pid; /* Pid du processus fils (i.e. le consommateur) */

/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU PERE                                  */
/* Le pid du processus fils est pass en paramtre.                     */
/************************************************************************/
void ProducteurCodeDuPere(int pid) {
  int i; /* Indice de boucle */

  printf("Producteur (pid = %d) : Lancement du code du processus pre...\n",
         getpid());
  printf("Producteur (pid = %d) : Le pid du fils est %d\n", getpid(), pid);
  printf("Producteur (pid = %d) : Ouverture en criture sur le tube 1...\n", getpid());

  /* Ouverture en criture du tube 1, attention l'ouverture est blocante */
  /* si pas de lecteur  l'autre bout du tube ! */
  fd1 = open(nom_tube1, O_WRONLY);
  fichier1 = fdopen(fd1, "w");
  printf("Producteur (pid = %d) : Tube 1 ouvert, par  lancer...\n", getpid());

  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
  /* En ce point une synchronisation entre le producteur et le          */
  /* consommateur a eu lieu : ds que le tube (vide pour l'instant)     */
  /* comporte (au moins) un metteur et (au moins) un rcepteur, ces    */
  /* processus sont dbloqus (ici seulement deux processus prvus).    */
  /* s'il y a dj des choses dans le tube, le rcepteur est            */
  /* immdiatement dbloqu.                                            */
  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

  printf("Producteur (pid = %d) : Attente de lecture sur le tube 2...\n", getpid());

  /* Ouverture en lecture du tube 2, attention l'ouverture est blocante */
  /* si pas d'metteur  l'autre bout du tube ! */
  fd2 = open(nom_tube2, O_RDONLY);
  fichier2 = fdopen(fd2, "r");
  printf("Producteur (pid = %d) : Tube 2 ouvert...\n", getpid());

  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
  /* En ce point une seconde synchronisation entre le producteur et le  */
  /* consommateur a eu lieu : ds que le tube (vide pour l'instant)     */
  /* comporte (au moins) un metteur et (au moins) un rcepteur, ces    */
  /* processus sont dbloqus (ici seulement deux processus prvus).    */
  /* s'il y a dj des choses dans le tube, le rcepteur est            */
  /* immdiatement dbloqu.                                            */
  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

  /* Dbut du "ping pong" : le producteur lance le ping ! */
  for (i = 1; i <= 10; i++) {
    printf("Producteur (pid = %d) : Emet PING %d sur tube 1 destin au consommateur\n", getpid(), i);
    fflush(stdout);
    fprintf(fichier1, "Producteur (pid = %d) : PING %d sur tube 1 vers consommateur\n", getpid(), i);
    /* Vous pouvez tester l'effet du fflush en commentant la ligne suivante */
    /* --> sans flush, pas d'criture effective dans le tube -> blocage */
    fflush(fichier1);
    sleep(1); /* Petit dlai pour ralentir le flux */
    printf("Producteur (pid = %d) : Attente de rception sur tube 2 ...\n", getpid());
    fflush(stdout);

    /* Ci-dessous il ne faut surtout pas boucler sur fgets pour lire */
    /* les informations depuis le tube 2 tant qu'il n'est pas vide */
    /* (via un "while"), comme dans le processus consommateur2tn car*/
    /* l'appel  fgets est "blocant" et le but principal du producteur */
    /* est de produire et non de se bloquer en attente de message */
    /* sinon -> DEADLOCK (essayez pour voir :-( !) */
    /* Ci dessous, il doit y avoir un et un seul message (infrieur  la */
    /* taille du buffer) et on ne fait que l'attendre */
    /* le ping pong est termin ou il y a une erreur (non teste ici) !*/
    if (fgets(buffer, 128, fichier2) != NULL) {
      printf("Producteur (pid = %d) : Reoit %s", getpid(), buffer);
      fflush(stdout);
    }
    else {
      printf("Producteur (pid = %d) : Plus rien  lire depuis le tube 2, arrt prmatur du consommateur !?\n", getpid());
      fflush(stdout);
      /* Je n'utilise pas exit() ici pour laisser le processus */
      /* se terminer "proprement" ! */
    }
  }

  printf("Producteur (pid = %d) : Fin du ping pong !\n", getpid());
  fflush(stdout);
  fclose(fichier2);
  close(fd2);
  fclose(fichier1);
  close(fd1);
}

/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU FILS                                  */
/* Ce processus est au dpart un "clone" (i.e. un fils obtenu via fork) */
/* du processus producteur. Le code de ce processus est ensuite remplac*/
/* par le code du processus consommateur suite  l'appel  "exec...()"  */
/************************************************************************/
void ProducteurCodeDuFils(void) {
  printf("Producteur (pid = %d) : Lancement du code du processus fils...\n",
         getpid());
  /* Allez on se lance ... un processus ! */
  if (execve("consommateur2tn", NULL, NULL)) {
    fprintf(stderr, "Producteur (pid = %d) : L'erreur '%s' s'est produite dans le du processus fils\n",
              getpid(), strerror(errno));
    fprintf(stderr, "Producteur (pid = %d) : lors du lancement du consommateur via execve()\n", getpid());
    fprintf(stderr, "Producteur (pid = %d) : Fin anormale du processus fils !\n", getpid());
    exit(EXIT_FAILURE);
  }
}

/************************************************************************/
/* PROGRAMME PRINCIPAL : CODE DU PROCESSUS PRODUCTEUR (PERE & FILS)     */
/************************************************************************/
int main(void) {
  printf("Producteur (pid = %d) : Lancement du programme producteur...\n", getpid());

  /* Ci-dessous, il faudrait tester l'existence des tubes plutt que */
  /* de les crer systmatiquement -> a faire en exercice !*/
  if (mkfifo(nom_tube1, 0644)) {
    fprintf(stderr, "Producteur (pid = %d) : L'erreur '%s' s'est produite lors de la cration du tube 1\n",
              getpid(), strerror(errno));
    fprintf(stderr, "Producteur (pid = %d) : On tente de continuer...\n", getpid());
  }
  if (mkfifo(nom_tube2, 0644)) {
    fprintf(stderr, "Producteur (pid = %d) : L'erreur '%s' s'est produite lors de la cration du tube 2\n",
              getpid(), strerror(errno));
    fprintf(stderr, "Producteur (pid = %d) : On tente de continuer...\n", getpid());
  }
  printf("Producteur (pid = %d) : Lancement du programme consommateur  partir du producteur...\n", getpid());

  /* En ce point, nous allons lancer le consommateur  partir du */
  /* en effectuant un fork() et en remplaant le code du fils ("clone" */
  /* du processus pre par dfaut) par le code du processus consommateur */
  /* grce  l'une des fonction exec...() qui effectue un remplacement */
  /* du code d'un processus ! */
  /* ATTENTION : par soucis de concision, tous les codes de retour et */
  /* toutes le prcautions ncessaires  l'criture d'un programme complet */
  /* et "robuste" n'ont pas t ajoute  ce programme */
  /* -> allez voir dans les pages de man (...) la documentation des */
  /* fonctions utilises et les prcautions  mettre en oeuvre */
  /* en guise d'exercice ! */

  pid = fork(); /* On clone le pre via fork */
  if (pid < 0) {
    fprintf(stderr, "Producteur (pid = %d) : L'erreur '%s' s'est produite lors de l'appel  fork()\n",
              getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }
  else if (pid > 0) {
    /*  Code du pre */
    ProducteurCodeDuPere(pid);
  }
  else {
    /* Code du fils */
    ProducteurCodeDuFils();
  }

  /* QUESTION SUBSIDIAIRE : QUI (pre et/ou fils) VA EXECUTER CETTE */
  /* PARTIE DU CODE EN FIN D'EXECUTION DE(S) PROGRAMME(S) */

  printf("Producteur/consommateur ? Fin du programme producteur excut par le processus pid = %d ...\n", getpid());
  printf("Producteur/consommateur ? (le processus parent de ce processus porte le numro ppid = %d)\n", getppid());
  fflush(stdout);
  return(EXIT_SUCCESS);
}
