#!/bin/bash

# Damien 20080305 : suppression des accents 
# SAS 20070719 : menus correctifs

##
# Le fichier de script SQL a ete obtenu a partir de druid
# Il y a quelques suppositions de dpart
#

# Variables
SQLDIR=.
BINDIR=base/bin
SCHEMA=schema/schema_cave.sql
DBNAME=cave
DBUSER=caviste
PG_BIN_DIR=/usr/local/pgsql/bin

export PGUSER=postgres
export DBNAME
# S'assurer que le client est compatible avec les locales
# On ne connait pas la locale de la base, a priori
# Il semblerait toutefois qu'un client en latin9 sache afficher
# des infos de base en UTF8
export PGCLIENTENCODING=latin9
SUPER_POSTGRES='sudo su postgres -c'

cd `dirname $0`

# si l'IP de la machine n'est pas precisee, on utilise 'localhost'
# L'interet etant d'utiliser l'information par la suite !
export PGHOST=${1:-localhost}

# Si la base existe, on la droppe avant de la creer
# Il faut que l'utilisateur courant soit un USER PG,
# on utilise donc postgres
RES=`$SUPER_POSTGRES "$PG_BIN_DIR/psql -l" | awk '{print $1}' | grep -i $DBNAME || echo "X"`
if [ "$RES" = $DBNAME ]; then
  echo "On droppe la base $DBNAME...";
  $SUPER_POSTGRES "$PG_BIN_DIR/dropdb $DBNAME";
fi

# On supprime l'utilisateur DBUSER s'il existe
RES=`echo "select rolname from pg_roles where rolname='$DBUSER'" | $SUPER_POSTGRES "$PG_BIN_DIR/psql -t template1" | awk '{print $1}'`
if [ "$RES" = "$DBUSER" ]; then
  echo "On droppe l'user $DBUSER..."
  $SUPER_POSTGRES "$PG_BIN_DIR/dropuser $DBUSER"
fi

# Creation PG user
USER_OPTS="--no-superuser --no-createrole --no-createdb $DBUSER --pwprompt"

echo "On cree l'user $DBUSER..."
$SUPER_POSTGRES "$PG_BIN_DIR/createuser $USER_OPTS"

echo "On cree la base $DBNAME..."
$SUPER_POSTGRES "$PG_BIN_DIR/createdb --owner $DBUSER $DBNAME"

echo "On ajoute le langage plpgsql a $DBNAME..."
$SUPER_POSTGRES "$PG_BIN_DIR/createlang plpgsql $DBNAME"

if [ -e $SCHEMA ] ; then

  # Creation du schema (TABLES, INDEX, SEQUENCES)
  echo "Creation du schema de la base de donnees"
  $PG_BIN_DIR/psql -q -U $DBUSER $DBNAME < $SCHEMA
 
  # creation des fonctions
  $PG_BIN_DIR/psql -q -U $DBUSER $DBNAME < $SQLDIR/fonctions/peuple_vin.sql
  $PG_BIN_DIR/psql -q -U $DBUSER $DBNAME < $SQLDIR/fonctions/peuple_stock.sql
  $PG_BIN_DIR/psql -q -U $DBUSER $DBNAME < $SQLDIR/fonctions/trous_stock.sql
  $PG_BIN_DIR/psql -q -U $DBUSER $DBNAME < $SQLDIR/fonctions/trous_vin.sql
 
  # Insertion des donnees
  $PG_BIN_DIR/psql -q -U $DBUSER $DBNAME < $SQLDIR/data/appellation_region.sql
  $PG_BIN_DIR/psql -q -U $DBUSER $DBNAME < $SQLDIR/data/data.sql

else 
  echo "Le fichier $SCHEMA est absent."
  echo "Le schema de la base n'a pas ete charge !"
fi
