/*----------------------------------------------------------------------*/
/* Fichier : producteur0shm.c						*/
/* Auteur  : D. Duvivier - LIL/ULCO					*/
/* Date    : 14/10/08							*/
/* Objectif: TP N2 (correction) M2/I2L Systme Rseaux Scurit	*/
/*	     Test d'utilisation d'un segment mmoire entre deux		*/
/*	     processus "lourds"						*/
/*	     Ce programme contient le producteur et le consommateur de	*/
/*	     donnes.							*/
/*----------------------------------------------------------------------*/
/* ATTENTION : dans cette version la "synchronisation" n'est pas assure*/
/* correctement car elle repose sur un sleep(15) dans le processus fils */ 
/*----------------------------------------------------------------------*/
/* Remarque : les appels "fflush(stdout)" ne servent  rien si ce n'est	*/
/* pour s'assurer que les messages soient bien affichs immdiatement	*/
/* en vitant les dlais ds  la buffrisation des e/s.		*/
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>

/* Variables globales -> a viter/limiter normalement (!) */
int pid;        /* Pid du processus fils (i.e. le consommateur) */
int shmid = -1; /* Indentificateur du segment mmoire partage */
		/* Commun au pre et au fils */

/* Acquisition d'un segment de mmoire partage (LM49) */
/* ATTENTION : le descripteur associ est retourn dans la variable */
/* globale/statique "shmid" */
int shm_init(char *filename, char project_id, int size) {
  if (shmid == -1) {
    /* Premier appel -> acquisition du segment */
    key_t k;
    shmid = shmget(k = ftok(filename, project_id), size, 0600 | IPC_CREAT);
    if (shmid <= -1) {
      /* Erreur lors de l'acquisition */
      perror("Impossible d'obtenir un segment mmoire");
      exit(EXIT_FAILURE);
    }
    else {
      /* OK : on a un segment ! */
      printf("Processus (pid = %d) : Segment mmoire allou correctement (shmid=%d)\n", getpid(), shmid);
      return (1); /* Signale nouveau segment  l'appelant */
    }
  }
  else {
    /* Un segment existe dj -> le rutiliser ! */
    printf("Processus (pid = %d) : Segment mmoire r-utilis (shmid=%d)\n", getpid(), shmid);
    return (0); /* Signale r-utilisation d'un segment  l'appelant */
  }
}

/* Suppression du segment mmoire dont l'id est stock dans la */
/* variable globale shmid. shmid et rinitialis  -1 par cette procdure */
void shm_destroy(int *shmid) {
  shmctl(*shmid, IPC_RMID, NULL);
  *shmid = -1; /* Scurit : indique segment non allou */
}

/* Gestion des signaux */
void signal_handler(int signal) {
  switch(signal) {
  case SIGUSR2:
    printf("Producteur (pid = %d) : Signal SIGUSR2 intercept !\n", getpid());
    sleep(1); /* Simule un traitement */
  break;
  case SIGHUP:
    printf("Producteur (pid = %d) : Signal SIGHUP intercept !\n", getpid());
  break;
  }
}


/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU PERE					*/
/* Le pid du processus fils est pass en paramtre.			*/
/************************************************************************/
void ProducteurCodeDuPere(int pid) {
  struct sigaction action;
  char *shmaddr = NULL; /* Adresse de "mappage" du segment */
		      /* (en ralit le pointeur est void *) */ 


  printf("Producteur (pid = %d) : Lancement du code du processus pre...\n",
         getpid());
  printf("Producteur (pid = %d) : Le pid du fils est %d\n", getpid(), pid);
  /* Seul le pre intercepte ces signaux ici,  la place de signal(), */
  /* on utilise cette fois une structure sigaction qui permet plus */
  /* de possibilits dans la gestions des signaux ... */
  /* les lignes suivantes remplacent en fait */
  /* signal(SIGUSR2, signal_handler); */

  action.sa_handler=signal_handler;
  sigemptyset(&(action.sa_mask));
  action.sa_flags = 0; /* Remplacer 0 par SA_ONESHOT pour une seule interception du signal */
  if (sigaction(SIGUSR2, &action, NULL) != 0) {
    fprintf(stderr, "Producteur (pid = %d) : Impossible de mettre en place le gestionnaire de signaux : %s\n", getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }

  printf("Producteur (pid = %d) : Le pre attache le segment de mmoire partage identifi par (shmid=%d) au segment de donnes...\n", getpid(), shmid);
  fflush(stdout);
  shmaddr = shmat(shmid, NULL, 0);    

  /* Ecriture d'une chane de caractres (<= 4096 caractres) */
  /* dans le segment mmoire ... */
  printf("Producteur (pid = %d) : Le pre crit dans le segment de mmoire partage identifi par (shmid=%d)...\n", getpid(), shmid);
  fflush(stdout);

  sprintf(shmaddr, "PING %d dans le segment mmoire\n", 1);

  printf("Producteur (pid = %d) : Le pre se met en attente pour 30 secondes...\n", getpid());
  fflush(stdout);
  sleep(30); /* Simule un traitement */

  /* On dtache le segment mmoire de l'espace d'adressage */
  printf("Producteur (pid = %d) : Le pre dtache le segment de mmoire partage identifi par (shmid=%d) au segment de donnes...\n", getpid(), shmid);
  fflush(stdout);
  shmdt(shmaddr);

  /* Une dernire fois nous attendons la fin du processus fils */
  /* (si ncessaire) : */
  printf("Producteur (pid = %d) : Attente de la fin du processus fils...\n", getpid());
  fflush(stdout);
  wait(NULL);

  printf("Producteur (pid = %d) : Fin du processus pre !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU FILS					*/
/* Ce processus est un "clone" (i.e. un fils obtenu via fork)		*/
/* du processus producteur.						*/
/************************************************************************/
void ProducteurCodeDuFils(void) {
  char *shmaddr = NULL; /* Adresse de "mappage" du segment */
		      /* (en ralit le pointeur est void *) */ 


  printf("Producteur (pid = %d) : Lancement du code du processus fils...\n",
         getpid());
  printf("Producteur (pid = %d) : Le fils attache le segment de mmoire partage identifi par (shmid=%d) au segment de donnes...\n", getpid(), shmid);
  fflush(stdout);
  shmaddr = shmat(shmid, NULL, 0);    

  printf("Producteur (pid = %d) : Le fils se met en attente pour 15 secondes...\n", getpid());
  fflush(stdout);
  sleep(15);
  printf("Producteur (pid = %d) : Le fils va lire dans le segment de mmoire partage identifi par (shmid=%d)...\n", getpid(), shmid);
  fflush(stdout);

  puts(shmaddr);

  /* On dtache le segment mmoire de l'espace d'adressage */
  printf("Producteur (pid = %d) : Le fils dtache le segment de mmoire partage identifi par (shmid=%d) au segment de donnes...\n", getpid(), shmid);
  fflush(stdout);
  shmdt(shmaddr);

  printf("Producteur (pid = %d) : Fin du processus fils !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* PROGRAMME PRINCIPAL : CODE DU PROCESSUS PRODUCTEUR (PERE & FILS)     */
/************************************************************************/
int main(int argc, char *argv[]) {
  printf("Producteur (pid = %d) : Lancement du programme producteur...\n", getpid());

  printf("Producteur (pid = %d) : Lancement du programme consommateur  partir du producteur...\n", getpid());

  /* On utilise le nom de l'excutable et le numro de projet (ici "1")*/
  /* pour calculer une clef via ftok et obtenir un descripteur de */
  /* segment mmoire unique */
  if (!shm_init(argv[0], 1, 4096)) {
    printf("Producteur (pid = %d) : Segment de mmoire partage dj attach ?\n", getpid());
    exit(EXIT_FAILURE);
  }

  /* En ce point nous allons lancer le consommateur  partir du */
  /* producteur en effectuant un fork() */
  /* ATTENTION : par soucis de concision, tous les codes de retour et */
  /* toutes le prcautions ncessaires  l'criture d'un programme complet*/
  /* et "robuste" n'ont pas t ajoute  ce programme */
  /* -> allez voir dans les pages de man (...) la documentation des */
  /* fonctions utilises et les prcautions  mettre en oeuvre */
  /* en guise d'exercice ! */

  pid = fork(); /* On clone le pre via fork */
  if (pid < 0) {
    fprintf(stderr, "Producteur (pid = %d) : L'erreur '%s' s'est produite lors de l'appel  fork()\n",
              getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }
  else if (pid > 0) {
    /*  Code du pre */
    ProducteurCodeDuPere(pid);
  }
  else {
    /* Code du fils */
    ProducteurCodeDuFils();
  }

  /* QUESTION SUBSIDIAIRE : QUI (pre et/ou fils) VA EXECUTER CETTE */
  /* PARTIE DU CODE EN FIN D'EXECUTION DE(S) PROGRAMME(S) */

  printf("Producteur/consommateur ? Fin du programme producteur excut par le processus pid = %d ...\n", getpid());
  printf("Producteur/consommateur ? (le processus parent de ce processus porte le numro ppid = %d)\n", getppid());
  fflush(stdout);
  return(EXIT_SUCCESS);
}
