/*----------------------------------------------------------------------*/
/* Fichier : producteur3semshm.c					*/
/* Auteur  : D. Duvivier - LIL/ULCO					*/
/* Date    : 14/10/08							*/
/* Objectif: TP N3 (correction) M2/I2L Systme Rseaux Scurit	*/
/*	     Test d'utilisation de deux smaphores entre trois		*/
/*	     processus "lourds" partageant un segment de mmoire.	*/
/*	     Dans cet exemple il y a deux producteurs et un consommateur*/
/*	     Les deux smaphores doivent assurer une lecture (et une	*/
/*	     seule) de chaque message crit. Ils assurent galement de	*/
/*	     bloquer le lecteur tant qu'il n'y a rien  lire.		*/
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	     Les messages des deux producteurs ne sont pas protgs	*/
/*	     ======================================================     */ 
/*	     ce qui fait que le dernier message dpos par le dernier	*/
/*	     des deux producteurs est lu deux fois par le consommateur ?*/
/*	     (le premier message dpos est perdu !?).			*/
/*??????????????????????????????????????????????????????????????????????*/
/* 	     LA PHRASE CI-DESSUS EST-ELLE VRAIE OU FAUSSE ??????????????*/
/*	     -> A VOUS DE ME LE DIRE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*??????????????????????????????????????????????????????????????????????*/
/*	     -> C'est une (premire tentative) de transmission de	*/
/*	     messages provenance de plusieurs producteurs mais est-ce   */
/*	     pas suffisant pour assurer la mmorisation de plusieurs	*/
/*	     messages dans le segment mmoire partag ?			*/
/*	     -> Pour la suite nous allons implmenter une file (FIFO)	*/
/*	     circulaire de messages dans le segment mmoire partag	*/
/*	     -> A VOUS DE LE FAIRE !					*/
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	     Ce programme lance les producteurs de donnes et le	*/
/*	     consommateur de donnes (via un ensemble de fork) et il se	*/
/*	     met en attente afin de supprimer les smaphores et le	*/
/*	     segment de mmoire lorsque tout est termin.		*/
/*----------------------------------------------------------------------*/
/* Remarque : les appels "fflush(stdout)" ne servent  rien si ce n'est	*/
/* pour s'assurer que les messages soient bien affichs immdiatement	*/
/* en vitant les dlais ds  la buffrisation des e/s.		*/
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>

	 /* La "constante" ci-dessous indique le nombre de smaphores */
	 /* dans le tableau de smaphores identifi par semid */
#define  NB_SEMAPHORES 2

	 /* Nombre de producteurs */
#define  NB_PRODUCTEURS 2
	 /* Numro de smaphore correspondant  BufferVide = 0 */
	 /* Numro de smaphore correspondant  BufferPlein = 1 */
#define	 BufferVide 0
#define	 BufferPlein 1

/* Variables globales -> a viter/limiter normalement (!) */
int conspid;    /* Pid du processus fils (i.e. le consommateur) */
int prodpid[NB_PRODUCTEURS]; /* Pid des processus producteurs */
int semid = -1; /* Identificateur du tableau de smaphores partag */
		/* Commun au pre et aux fils */
int shmid = -1; /* Identificateur du segment mmoire partage */
		/* Commun au pre et aux fils */

/* Acquisition d'un segment de mmoire partage (LM49) */
/* ATTENTION : le descripteur associ est retourn dans la variable */
/* globale/statique "shmid" */
int shm_init(char *filename, char project_id, int size) {
  if (shmid == -1) {
    /* Premier appel -> acquisition du segment */
    key_t k;
    shmid = shmget(k = ftok(filename, project_id), size, 0600 | IPC_CREAT);
    if (shmid <= -1) {
      /* Erreur lors de l'acquisition */
      perror("Impossible d'obtenir un segment mmoire");
      exit(EXIT_FAILURE);
    }
    else {
      /* OK : on a un segment ! */
      printf("Processus (pid = %d):Segment mmoire allou correctement (shmid=%d)\n", getpid(), shmid);
      return (1); /* Signale nouveau segment  l'appelant */
    }
  }
  else {
    /* Un segment existe dj -> le rutiliser ! */
    printf("Processus (pid = %d):Segment mmoire r-utilis (shmid=%d)\n", getpid(), shmid);
    return (0); /* Signale r-utilisation d'un segment  l'appelant */
  }
}

/* Suppression du segment mmoire dont l'id est stock dans la*/
/* variable globale shmid. shmid et rinitialis  -1 par cette procdure */
void shm_destroy(void) {
  shmctl(shmid, IPC_RMID, NULL);
  shmid = -1; /* Scurit : indique segment non allou */
}


/* Acquisition de NB_SEMAPHORES smaphore(s) (LM36) */
/* ATTENTION : le descripteur associ est retourn dans la variable */
/* globale/statique "semid" */
int sem_init(char *filename, char project_id) {
  key_t k;

  if (semid == -1) {
    /* Accder au tableau de smaphores s'il existe dj */
    semid = semget(k = ftok(filename, project_id), NB_SEMAPHORES, 0600);
    if (semid == -1) {
      /* Premier appel -> acquisition du tableau de smaphores */
      semid = semget(k = ftok(filename, project_id), NB_SEMAPHORES, 0600 | IPC_CREAT);
      if (semid <= -1) {
        /* Erreur lors de l'acquisition */
        perror("Impossible d'obtenir des smaphores");
        exit(EXIT_FAILURE);
      }
      else {
        /* OK : on a des smaphores ! */
        printf("Processus (pid = %d):Tableau de smaphores allou correctement (semid=%d)\n", getpid(), semid);
        return (1); /* Signale nouveau tableau de smaphores  l'appelant */
      }
    }
    else {
      /* Le tableau de smaphores existe dj et c'est le premier appel  sem_init -> le rutiliser ! */
      printf("Processus (pid = %d):Tableau de smaphores r-utilis (semid=%d)\n", getpid(), semid);
      return (0); /* Signale r-utilisation des smaphores  l'appelant */
    }
  }
  else {
    /* Le tableau de smaphores existe dj et ce n'est pas le premier appel  sem_init -> le rutiliser ! */
    printf("Processus (pid = %d):Tableau de smaphores r-utilis (semid=%d)\n", getpid(), semid);
    return (-1); /* Signale r-utilisation des smaphores  l'appelant */
                 /* et appels multiples de sem_init */
  }
}

/* Suppression du tableau de smaphores dont l'id est stock dans la*/
/* variable globale semid. semid est reinitialise a -1 par cette procedure */
void sem_destroy(void) {
  semctl(semid, 0, IPC_RMID, 0);
  semid = -1; /* Scurit : indique smaphores non allou */
}

/* Fixe la valeur initiale du smaphore */
void sem_setvalue(int sem, int val) {
#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
       /* l'union semun est dfinie par l'inclusion de <sys/sem.h> */
       union  semun arg;
#else
       /* d'aprs X/OPEN il faut la dfinir nous-mmes */
       union semun {
             int val;                  /* valeur pour SETVAL */
             struct semid_ds *buf;     /* buffer pour IPC_STAT, IPC_SET */
             unsigned short *array;    /* table  pour GETALL, SETALL */
                                       /* Spcificit Linux : */
             struct seminfo *__buf;    /* buffer pour IPC_INFO */
       } arg;
#endif

  arg.val = val; /* Fixe la valeur initiale du smaphore */
  if (semctl(semid, sem, SETVAL, arg) <= -1) {
    perror("Impossible de fixer la valeur initiale du smaphore");
    exit(EXIT_FAILURE);
  }
}

/* P(smaphore) */
int sem_askresource (int sem) {
  int r;
  struct sembuf s;
  s.sem_num = sem;
  s.sem_op = -1;
  s.sem_flg = 0;
  if ((r = semop(semid, &s, 1)) <= -1) {
    perror("Impossible de rserver la ressource");
    exit(EXIT_FAILURE);
  }
  return r;
}

/* V(smaphore) */
int sem_releaseresource (int sem) {
  int r;
  struct sembuf s;
  s.sem_num = sem;
  s.sem_op = 1;
  s.sem_flg = 0;
  if ((r = semop(semid, &s, 1)) <= -1) {
    perror("Impossible de librer la ressource");
    exit(EXIT_FAILURE);
  }
  return r;
}

/************************************************************************/
/* PROCESSUS PRODUCTEUR							*/
/************************************************************************/
void Producteur(void) {
  int i; /* Indice de boucle */
  char *shmaddr = NULL; /* Adresse de "mappage" du segment */
		        /* (en ralit le pointeur est void *) */ 

  printf("Producteur(pid = %d):Lancement du code du processus\n",
         getpid());
  printf("Producteur(pid = %d):Attache le segment partag (shmid=%d) au segment de donnes\n", getpid(), shmid);
  fflush(stdout);
  shmaddr = shmat(shmid, NULL, 0);    

  printf("Producteur(pid = %d):Lancement du ping pong\n", getpid());
  fflush(stdout);

  for (i = 1; i <= 10; i++) {
    printf("Producteur(pid = %d):Demande d'accs en criture P(sem[BufferVide])\n", getpid());
    fflush(stdout);

    sem_askresource(BufferVide); /* P(semaphore[BufferVide]) */

    /* Ecriture d'une chane de caractres (<= 4096 caractres) */
    /* dans le segment mmoire  */
    printf("Producteur(pid = %d):Ecriture #%d dans le segment partag (shmid=%d)\n", getpid(), i, shmid);
    fflush(stdout);

    sprintf(shmaddr, "PING %d du processus %d dans le segment mmoire\n", i, getpid());

    sleep(1); /* Une seconde en section critique */

    printf("Producteur(pid = %d):Quitte section critique & autorise la lecture via smaphore BufferPlein : V(sem[BufferPlein])\n", getpid());    fflush(stdout);
    sem_releaseresource(BufferPlein); /* V(semaphore[BufferPlein]) */

    printf("Producteur(pid = %d):Hors de la section critique\n", getpid());    fflush(stdout);

    sleep(1); /* Une seconde hors de section critique */
  }

  /* On dtache le segment mmoire de l'espace d'adressage */
  printf("Producteur(pid = %d):Dtache le segment partag (shmid=%d) au segment de donnes\n", getpid(), shmid);
  fflush(stdout);
  shmdt(shmaddr);

  printf("Producteur(pid = %d):Fin du processus !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* PROCESSUS CONSOMMATEUR						*/
/************************************************************************/
void Consommateur(void) {
  int i; /* Indice de boucle */
  char *shmaddr = NULL; /* Adresse de "mappage" du segment */
		      /* (en ralit le pointeur est void *) */ 

  printf("Consommateur(pid=%d):Lancement du code du processus\n",
         getpid());
  fflush(stdout);

  printf("Consommateur(pid=%d):Attache le segment partag (shmid=%d) au segment de donnes\n", getpid(), shmid);
  fflush(stdout);
  shmaddr = shmat(shmid, NULL, 0);    

  printf("Consommateur(pid=%d):Lancement du ping pong\n",
        getpid());
  fflush(stdout);

  for (i = 1; i <= 10*NB_PRODUCTEURS; i++) {
    printf("Consommateur(pid=%d):Demande d'accs  la section critique P(sem(BufferPlein))\n", getpid());
    fflush(stdout);

    sem_askresource(BufferPlein); /* P(semaphore[BufferPlein]) demande de lecture */

    printf("Consommateur(pid=%d):Lecture #%d dans le segment partag (shmid=%d)\n", getpid(), i, shmid);
    fflush(stdout);

    printf("Consommateur(pid=%d):Chane lue=%s", getpid(), shmaddr);
    fflush(stdout);

    usleep(200000); /* 2/10 secondes en section critique */

    printf("Consommateur(pid=%d):Quitte section critique & autorise l'criture via smaphore BufferVide : V(sem[BufferVide])\n", getpid());    fflush(stdout);
    sem_releaseresource(BufferVide); /* V(semaphore[BufferVide]) */

    printf("Consommateur(pid=%d):Hors de la section critique\n", getpid());    fflush(stdout);
    usleep(200000); /* 2/10 secondes hors de section critique */
  }

  /* On dtache le segment mmoire de l'espace d'adressage */
  printf("Consommateur(pid=%d):Dtache le segment partag (shmid=%d) au segment de donnes\n", getpid(), shmid);
  fflush(stdout);
  shmdt(shmaddr);

  printf("Consommateur(pid=%d):Fin du processus !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/*			 PROGRAMME PRINCIPAL				*/
/************************************************************************/
int main(int argc, char *argv[]) {
  int i; /* Indice de boucle */

  printf("ProcessPre (pid=%d):Lancement du processus pre\n", getpid());

  /* On utilise le nom de l'excutable et le numro de projet (ici "1")*/
  /* pour calculer une clef via ftok et obtenir un descripteur de */
  /* segment mmoire unique */
  if (!shm_init(argv[0], 1, 4096)) {
    printf("ProcessPre (pid=%d):segment partag dj attach ?\n", getpid());
    exit(EXIT_FAILURE);
  }

  /* On utilise le nom de l'excutable et le numro de projet (ici "1")*/
  /* pour calculer une clef via ftok et obtenir un descripteur de */
  /* tableau de smaphores unique */
  if (!sem_init(argv[0], 1)) {
    printf("ProcessPre (pid=%d):Le tableau de smaphores existe dj\n", getpid());
    /* inutile de faire exit(EXIT_FAILURE) ici ce n'est pas une erreur*/
  }

  /* On fixe la valeur initiale du smaphore NBufferVide  la valeur 1 */
  sem_setvalue(BufferVide, 1); /* Smaphore utilis pour l'criture */
  sem_setvalue(BufferPlein, 0); /* Smaphore utilis pour la lecture (bloqu au dpart) */

  /* En ce point nous allons lancer le consommateur  partir du */
  /* producteur en effectuant un fork() */
  /* ATTENTION : par soucis de concision, tous les codes de retour et */
  /* toutes le prcautions ncessaires  l'criture d'un programme complet*/
  /* et "robuste" n'ont pas t ajoute  ce programme */
  /* -> allez voir dans les pages de man () la documentation des */
  /* fonctions utilises et les prcautions  mettre en oeuvre */
  /* en guise d'exercice ! */

  conspid = fork(); /* On clone le pre pour obtenir le consommateur */
  if (conspid < 0) {
    fprintf(stderr, "ProcessPre (pid=%d):L'erreur '%s' s'est produite lors de l'appel  fork() pour crer le consommateur\n",
              getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }
  else if (conspid > 0) {
    /* Code du pre */
    /* Le pre va lancer tous les producteurs comme des fils. */
    /* C'est un peu diffrent de ce qui tait effectu prcdemment : */
    /* Le pre (c'est--dire le processus initialement lanc depuis le */
    /* se comporte comme un serveur : il lance toutes le tches de calcul*/
    /* dans des processus fils (producteurs et consommateur) et se met*/
    /* en attente des vnement de terminaison (...) des processus fils */
    printf("ProcessPre (pid=%d):Lancement des producteurs\n", getpid());
    fflush(stdout);
    for (i=0; i<NB_PRODUCTEURS; i++) {
      prodpid[i] = fork(); /* On clone le pre pour obtenir 1 producteur */

      if (prodpid[i] < 0) {
        fprintf(stderr, "ProcessPre (pid=%d):L'erreur '%s' s'est produite lors de l'appel  fork() pour crer un producteur\n",
              getpid(), strerror(errno));
        exit(EXIT_FAILURE);
      }
      else if (prodpid[i] == 0) {
        /* Code du fils : il faut lancer le producteur ... */
        Producteur();
        /* et ne surtout pas oublier de stopper la boucle pour ! */
        /* sinon cela va lancer de producteurs en cascade !!! */
        i = -1; /* Indique boucle termine par break dans un fils */
        break; /* -> fin de boucle */
      }
      /* else/sinon nous sommes dans le processus pre, dans lequel la */
      /* boucle for/pour doit se poursuivre pour lancer les */
      /* producteurs ...*/

    } /* for */
    /* Suite du code excut par le pre ET par les fils producteurs */
    /* Si i > 0 alors il s'agit du processus pre */
    if (i > 0) {
      /* Il faut attendre la fin des NB_PRODUCTEURS + 1 fils ...*/
      printf("ProcessPre (pid=%d):Mise en attente du pre\n", getpid());
      fflush(stdout);
      for (i=0; i<=NB_PRODUCTEURS; i++) {
        /* Ci-dessous il faudrait intercepter les codes de retour et */
        /* vrifier si la sortie de wait est bien due  l'arrt d'un */
        /* processus fils -> A faire en exercice ! */
        wait(NULL);
        printf("ProcessPre (pid=%d):Un processus fils vient de se terminer...\n", getpid());
        fflush(stdout);
      }
      printf("ProcessPre (pid=%d):Tous les processus fils sont termins\n", getpid());
      /* Il faut faire le mnage ... */
      printf("ProcessPre (pid=%d):Destruction des smaphores\n", getpid());
      fflush(stdout);
      sem_destroy();
      printf("ProcessPre (pid=%d):Destruction du segment de mmoire partag\n", getpid());
      fflush(stdout);
      shm_destroy();
    } /* if (i > 0) */
  }
  else {
    /* Code du fils consommateur */
    Consommateur();
  }

  /* QUESTION SUBSIDIAIRE : QUI (pre et/ou fils) VA EXECUTER CETTE */
  /* PARTIE DU CODE EN FIN D'EXECUTION DE(S) PROGRAMME(S) */

  printf("Producteur/consommateur ? Fin du programme excut par le processus pid = %d \n", getpid());
  printf("Producteur/consommateur ? (le processus parent de ce processus porte le numro ppid = %d)\n", getppid());
  fflush(stdout);
  return(EXIT_SUCCESS);
}
