/*----------------------------------------------------------------------*/
/* Fichier : producteursig.c						*/
/* Auteur  : D. Duvivier - LIL/ULCO					*/
/* Date    : 22/09/09							*/
/* Objectif: TP N3 (correction) M2/I2L Systme Rseaux Scurit	*/
/*	     Test de deux signaux entre deux processus "lourds"		*/
/*	     Ce programme est le producteur de donnes.			*/
/*	     Il se charge de lancer le consommateur de donnes.		*/
/*----------------------------------------------------------------------*/
/* Remarque : les appels "fflush(stdout)" ne servent  rien si ce n'est	*/
/* pour s'assurer que les messages soient bien affichs immdiatement	*/
/* en vitant les dlais ds  la buffrisation des e/s.		*/
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>

/* Variables globales -> viter/limiter normalement (!) */
int pid; /* Pid du processus fils (i.e. le consommateur) */


/* Gestion des signaux */
void signal_handler(int signal) {
  switch(signal) {
  case SIGUSR2:
    printf("Producteur (pid = %d) : Signal SIGUSR2 intercept !\n", getpid());
    sleep(1); /* Simule un traitement */
  break;
  case SIGHUP:
    printf("Producteur (pid = %d) : Signal SIGHUP intercept !\n", getpid());
  break;
  }
}


/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU PERE					*/
/* Le pid du processus fils est pass en paramtre.			*/
/************************************************************************/
void ProducteurCodeDuPere(int pid) {
  int i; /* Indice de boucle */
  /* int code; */
  struct sigaction action;

  printf("Producteur (pid = %d) : Lancement du code du processus pre...\n",
         getpid());
  printf("Producteur (pid = %d) : Le pid du fils est %d\n", getpid(), pid);
  /* Seul le pre intercepte ces signaux ici,  la place de signal(), */
  /* on utilise cette fois une structure sigaction qui permet plus */
  /* de possibilits dans la gestions des signaux ... */
  /* les lignes suivantes remplacent en fait */
  /* signal(SIGUSR2, signal_handler); */

  action.sa_handler=signal_handler;
  sigemptyset(&(action.sa_mask));
  action.sa_flags = 0; /* Remplacer 0 par SA_ONESHOT pour une seule interception du signal */
  if (sigaction(SIGUSR2, &action, NULL) != 0) {
    fprintf(stderr, "Producteur (pid = %d) : Impossible de mettre en place le gestionnaire de signaux : %s\n", getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }

  sleep(3);

  printf("Producteur (pid = %d) : Lancement du ping pong...\n",
        getpid());
  fflush(stdout);

  for (i = 1; i <= 10; i++) {
    printf("Producteur (pid = %d) : Emet PING %d via SIGUSR1 destin au consommateur\n", getpid(), i);
    fflush(stdout);
    kill(pid, SIGUSR1); /* Envoit le signal SIGUSR1 au fils */

    printf("Producteur (pid = %d) : Blocage #%d sur wait()...\n", getpid(), i);    fflush(stdout);

    wait(NULL);
  }
  /* Une dernire fois nous attendons la fin du processus fils */
  /* (si ncessaire) : */
  printf("Producteur (pid = %d) : Attente de la fin du processus fils...\n", getpid());
  fflush(stdout);
  wait(NULL);

  printf("Producteur (pid = %d) : Fin du processus pre !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU FILS					*/
/* Ce processus est au dpart un "clone" (i.e. un fils obtenu via fork)	*/
/* du processus producteur. Le code de ce processus est ensuite remplac*/
/* par le code du processus consommateur suite  l'appel  "exec...()"	*/
/************************************************************************/
void ProducteurCodeDuFils(void) {
  printf("Producteur (pid = %d) : Lancement du code du processus fils...\n",
         getpid());
  /* Allez on se lance ... un processus ! */
  if (execve("consommateursig", NULL, NULL)) {
    fprintf(stderr, "Producteur (pid = %d) : L'erreur '%s' s'est produite dans le du processus fils\n",
              getpid(), strerror(errno));
    fprintf(stderr, "Producteur (pid = %d) : lors du lancement du consommateur via execve()\n", getpid());
    fprintf(stderr, "Producteur (pid = %d) : Fin anormale du processus fils !\n", getpid());
    exit(EXIT_FAILURE);
  }
}

/************************************************************************/
/* PROGRAMME PRINCIPAL : CODE DU PROCESSUS PRODUCTEUR (PERE & FILS)     */
/************************************************************************/
int main(void) {
  printf("Producteur (pid = %d) : Lancement du programme producteur...\n", getpid());

  printf("Producteur (pid = %d) : Lancement du programme consommateur  partir du producteur...\n", getpid());

  /* En ce point nous allons lancer le consommateur  partir du */
  /* en effectuant un fork() et en remplaant le code du fils ("clone" */
  /* du processus pre par dfaut) par le code du processus consommateur */
  /* grce  l'une des fonction exec...() qui effectue un remplacement */
  /* du code d'un processus ! */
  /* ATTENTION : par soucis de concision, tous les codes de retour et */
  /* toutes le prcautions ncessaires  l'criture d'un programme complet*/
  /* et "robuste" n'ont pas t ajoute  ce programme */
  /* -> allez voir dans les pages de man (...) la documentation des */
  /* fonctions utilises et les prcautions  mettre en oeuvre */
  /* en guise d'exercice ! */

  pid = fork(); /* On clone le pre via fork */
  if (pid < 0) {
    fprintf(stderr, "Producteur (pid = %d) : L'erreur '%s' s'est produite lors de l'appel  fork()\n",
              getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }
  else if (pid > 0) {
    /*  Code du pre */
    ProducteurCodeDuPere(pid);
  }
  else {
    /* Code du fils */
    ProducteurCodeDuFils();
  }

  /* QUESTION SUBSIDIAIRE : QUI (pre et/ou fils) VA EXECUTER CETTE */
  /* PARTIE DU CODE EN FIN D'EXECUTION DE(S) PROGRAMME(S) */

  printf("Producteur/consommateur ? Fin du programme producteur excut par le processus pid = %d ...\n", getpid());
  printf("Producteur/consommateur ? (le processus parent de ce processus porte le numro ppid = %d)\n", getppid());
  fflush(stdout);
  return(EXIT_SUCCESS);
}
