/*----------------------------------------------------------------------*/
/* Fichier : tstpingpong.c						*/
/* Auteur  : D. Duvivier - LIL/ULCO					*/
/* Date    : 21/02/08							*/
/* Objectif: TP/TD N1 L2/Info Systmes d'Exploitation			*/
/*	     Test d'une synchronisation en alternat de deux processus	*/
/*	     -> "ping pong"						*/
/*----------------------------------------------------------------------*/
/*
#############################################################################
#This program is free software; you can redistribute it and/or modify       #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#############################################################################
*/
#include <ddtools.h>

#define SemaphorePING	semaphore0
#define SemaphorePONG	semaphore1

/* Variables globales -> a viter/limiter normalement (!) */
pid_t pid; /* Process identifier du fils ou 0 */

/************************************************************************/
/* CODE D'INITIALISATION SPECIFIQUE A UN EXERCICE PARTICULIER :		*/
/* A APPELER AVANT LE/LES APPEL(s) A FORK :				*/
/************************************************************************/
void CodeDInitialisation(int argc, char *argv[]) {
  DD_init(argc, argv, 2); /* Initialisation pour 2 smaphores AVANT LE FORK !!! */
  DD_sem_setvalue(SemaphorePING, 1); /* Initialise smaphore PING  1 */
  DD_sem_setvalue(SemaphorePONG, 0); /* Initialise smaphore PING  0 */
}

/************************************************************************/
/* CODE DU PERE / PING: Le pid du processus fils est pass en paramtre.*/
/************************************************************************/
void CodeDuPere(int pid) {
  int i;  /* Indice de boucle */

  DD_pidmsg2("PERE", "Lancement du code du processus PING...");
  /* Tout est en place, nous gnrons simplement une petite attente */
  sleep(2);
  DD_pidmsg2("PERE", "Lancement du ping pong...");
  /* On amorce le ping-pong depuis le pre puis on dbloque le fils... */
  for (i = 1; i <= 10; i++) {
    Down(SemaphorePING);
    printf("PING        :pid=%d:Iteration %d\n", getpid(), i);
    fflush(stdout);
    usleep(200000); /* 2/10 secondes en section critique */
    Up(SemaphorePONG);
  }
  DD_pidmsg2("PERE", "Fin du code du processus pre !");
}

/************************************************************************/
/* CODE DU FILS	/ PONG							*/
/************************************************************************/
void CodeDuFils(void) {
  int i = 0; /* Indice de boucle */

  DD_pidmsg2("FILS", "Lancement du code du processus PONG...");
  for (i = 1; i <= 9; i++) {
    Down(SemaphorePONG);
    printf("PONG        :pid=%d:Iteration %d\n", getpid(), i);
    fflush(stdout);
    usleep(200000); /* 2/10 secondes en section critique */
    Up(SemaphorePING);
  }
  /* Fin du dernier change sans "Up" sur SemaphorePING */
  Down(SemaphorePONG);
  printf("PONG        :pid=%d:Iteration %d\n", getpid(), i);
  fflush(stdout);
  usleep(200000); /* 2/10 secondes en section critique */
  DD_pidmsg2("FILS", "Fin du code du processus fils");
}

/************************************************************************/
/* PROGRAMME PRINCIPAL : LANCEMENT DES PROCESSUS PERE ET FILS)		*/
/*----------------------------------------------------------------------*/
/* Ce programme doit garantir que le processus pre soit le dernier	*/
/* processus actif (pour appeler DD_close().				*/
/************************************************************************/
int main(int argc, char *argv[]) {
  DD_pidmsg2("PERE", "Lancement du programme...");
  /* Initialisation(s) avant fork() */
  CodeDInitialisation(argc, argv);
  pid = fork(); /* On clone le pre via fork */
  if (pid < 0) {
    DD_piderr2("PERE", "Erreur lors de l'appel  fork()");
    exit(EXIT_FAILURE); /* fork() a echoue -> fin du programme */
  }
  else if (pid > 0) {
    /* Code du pre */
    CodeDuPere(pid);
    DD_pidmsg2("PERE", "Bloqu en attente de la terminaison de ses fils...");
    /* Bloquer le processus pre tant qu'au moins un fils est actif : */
    DD_waitallchilds();
    DD_close(); /* Seul le pre (dernier process actif) doit appeler DD_close() */
  }
  else {
    /* Code du fils */
    CodeDuFils();
  }
  /* Le pre et le fils vont passer ici en fin de programme sauf erreur */
  DD_pidmsg2("PEREouFILS", "Fin du programme");
  return(EXIT_SUCCESS);
}
